package de.beocode.bestbefore

import android.app.Application
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.viewModels
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.height
import androidx.compose.material.*
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import dagger.hilt.android.AndroidEntryPoint
import dagger.hilt.android.HiltAndroidApp
import de.beocode.bestbefore.app.BestBeforeApp
import de.beocode.bestbefore.data.AppData
import de.beocode.bestbefore.data.FoodItem
import de.beocode.bestbefore.data.appDataStore
import de.beocode.bestbefore.data.dataDataStore
import de.beocode.bestbefore.ui.theme.BestBeforeTheme
import de.beocode.bestbefore.viewmodels.MainViewModel
import de.beocode.bestbefore.viewmodels.interfaces.LocalUserState
import kotlinx.coroutines.flow.first

@HiltAndroidApp
class App : Application()

@AndroidEntryPoint
class MainActivity : ComponentActivity() {
    private val userState by viewModels<MainViewModel>()
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContent {
            BestBeforeTheme {
                CompositionLocalProvider(LocalUserState provides userState) {
                    BestBeforeApp()

                    var migration by remember { mutableStateOf(false) }
                    LaunchedEffect(key1 = Unit) {
                        migrateData { migration = it }
                    }
                    if (migration) {
                        AlertDialog(
                            title = {
                                Text(
                                    text = "Data migration",
                                    style = MaterialTheme.typography.h5
                                )
                            },
                            text = {
                                Column {
                                    Text(
                                        text = "Data is copied.\nThis may take a moment.",
                                        fontSize = 20.sp
                                    )
                                    Spacer(modifier = Modifier.height(5.dp))
                                    LinearProgressIndicator()
                                }
                            },
                            buttons = { },
                            onDismissRequest = { }
                        )
                    }
                }
            }
        }
    }

    private suspend fun migrateData(update: (Boolean) -> Unit) {
        val oldData = dataDataStore.data.first()
        if (oldData.foodList.isNotEmpty()) {
            runOnUiThread {
                update(true)
            }

            val newData = AppData()

            userState.setWarningDays(oldData.warningDays)

            val foodList = newData.foodList.toMutableList()
            for (food in oldData.foodList) {
                foodList.add(
                    FoodItem(
                        id = food.id,
                        name = food.name,
                        ts = food.dateTS,
                        amount = food.amount,
                        place = food.destination
                    )
                )
            }
            foodList.sortBy { it.ts }
            try {
                appDataStore.updateData { AppData ->
                    AppData.copy(
                        foodList = foodList
                    )
                }
                dataDataStore.updateData { Data ->
                    Data.toBuilder()
                        .clearFood()
                        .build()
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            val nameList = newData.nameSuggestions.toMutableList()
            val placeList = newData.placeSuggestions.toMutableList()
            for (name in oldData.nameList) {
                nameList.add(name)
            }
            for (place in oldData.destinationList) {
                placeList.add(place)
            }
            try {
                appDataStore.updateData { AppData ->
                    AppData.copy(
                        nameSuggestions = nameList,
                        placeSuggestions = placeList
                    )
                }
                dataDataStore.updateData { Data ->
                    Data.toBuilder()
                        .clearName()
                        .clearDestination()
                        .build()
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            update(false)
        }
    }
}